#!/usr/bin/env python

import os, sys, codecs

import pygame
import pygame.mixer

from ocempgui.widgets import *
from ocempgui.widgets.Constants import *
from ocempgui.widgets.components import *

HSIZE=320
VSIZE=240
HSCROLL_SCALE=(HSIZE-100)

def playSound(filename):
	if os.path.exists(filename):
		pygame.mixer.Sound(filename).play()
		return
	for extension in 'spx', 'ogg', 'mp3', 'wav':
		if os.path.exists('%s.%s' % (filename, extension)):
			try:
				pygame.mixer.Sound('%s.%s' % (filename, extension)).play()
				return
			except Exception, e:
				print 'WARNING: %s playing %s' % (e, filename)
	raise Exception('Could not find sound file %s' % filename)

class ParallelFileHandler(object):
	def __init__(self):
		self.basePath = None
		self.soundPath = None
		self.filename = None
		self.rawLines = []
		self.data = []
	def load(self, filename):
		self.rawLines = []
		self.data = []
		f = codecs.open(filename, 'r', 'utf-16')
		self.filename = filename
		self.basePath = os.path.dirname(filename)
		self.soundPath = os.path.join(self.basePath, f.readline().strip())
		
		soundFile, curText = None, None
		for line in f:
			if line[0] == '+':
				if curText is None:
					raise Exception('Attempt to append to empty buffer')
				curText += line[1:]
				continue
			if line[0] == '`':
				(soundFile, line) = line[1:].split('`', 1)
			if curText is not None:
				self.rawLines.append((soundFile, curText))
				soundFile, curText = None, None
			curText = line

		if curText is not None:
			self.rawLines.append((soundFile, curText))
		self.data = list(self._makePairs())
	def _makePairs(self):
		curData = [[]]
		for item in self.rawLines:
			if item[0] != None:
				curData[0].append(item[0])
			curData.append(item[1])
			if len(curData) == 3:
				yield curData
				curData = [[]]
		if len(curData) != 1:
			raise Exception('Partial content at end of file: %s' % curData)
	def __len__(self):
		return len(self.data)
	def __getitem__(self, index):
		return self.data[index]

class KeyListener(object):
	def __init__(self, mainScreen):
		self.mainScreen = mainScreen
	def notify(self, event):
		if event.data.key == pygame.K_PAGEUP:
			self.mainScreen.modifyPos(-1)
			event.handled = True
		elif event.data.key == pygame.K_PAGEDOWN:
			self.mainScreen.modifyPos(1)
			event.handled = True
		elif event.data.key == pygame.K_SPACE:
			self.mainScreen.speak()
			event.handled = True
		elif event.data.key == pygame.K_HOME:
			self.mainScreen.modifyPos(0, absolute=True)
			event.handled = True
		elif event.data.key == pygame.K_END:
			self.mainScreen.modifyPos(len(self.mainScreen.data), absolute=True)
			event.handled = True

class MainScreen(object):
	def __init__(self, renderer):
		self.renderer = renderer
		base.GlobalStyle.styles['default']['font']['name'] = 'tuffy.ttf'
		base.GlobalStyle.styles['default']['font']['size'] = 12
		self.data = ParallelFileHandler()
		self.currentItemIdx = 0
	def init(self):
		self.status = StatusBar()
		self.status.push_tip('Load a module to begin')
		self.status.size = (HSIZE, self.status.size[1])
		self.status.position = (0, 240 - self.status.size[1])
		self.renderer.add_widget(self.status)
		
		self.widget_frame = VFrame()
		self.widget_frame.spacing = 3
		self.widget_frame.border = BORDER_NONE
		
		self.button_frame = HFrame()
		self.button_frame.spacing = 3
		self.button_frame.align = ALIGN_TOP
		self.button_frame.border = BORDER_NONE
		self.widget_frame.add_child(self.button_frame)
		
		self.load_button = Button('#Load Module...')
		self.load_button.connect_signal(SIG_CLICKED, self.fileDialog)
		self.button_frame.add_child(self.load_button)
		#self.renderer.add_widget(self.load_button)
		
		## change me later w/ set_maximum; set current position with set_value.
		self.scrollbar = HScrollBar(HSCROLL_SCALE, HSCROLL_SCALE)
		self.scrollbar.size=(HSCROLL_SCALE,self.scrollbar.size[1])
		self.scrollbar.step=HSCROLL_SCALE
		self.scrollbar.connect_signal(SIG_VALCHANGED, self.scrollbarMoved)
		self.button_frame.add_child(self.scrollbar)
		
		self.word1_frame = VFrame()
		self.word1_frame.size=(HSIZE-6,80)
		self.widget_frame.add_child(self.word1_frame)
		
		self.word1_label = Label('')
		self.word1_label.multiline = True
		self.word1_frame.add_child(self.word1_label)
		
		self.word2_frame = VFrame()
		self.word2_frame.size=(HSIZE-6,80)
		self.widget_frame.add_child(self.word2_frame)
		
		self.word2_label = Label('')
		self.word2_label.multiline = True
		self.word2_frame.add_child(self.word2_label)
		
		self.renderer.add_widget(self.widget_frame)
		self.renderer.add_high_priority_object(KeyListener(self), SIG_KEYDOWN)
	def fileDialog(self):
		buttons = [Button('#OK'), Button('#Cancel')]
		buttons[0].size = (80, buttons[0].size[1])
		buttons[1].size = (80, buttons[1].size[1])
		results = [DLGRESULT_OK, DLGRESULT_CANCEL]
		
		dialog = FileDialog('Select file to load', buttons, results)
		dialog.depth = 1
		dialog.position = 2,2
		dialog.filelist.selectionmode = SELECTION_SINGLE
		dialog.connect_signal(SIG_DIALOGRESPONSE, self.fileDialogDone, dialog)
		self.renderer.add_widget(dialog)
	def updateWords(self):
		if self.currentItemIdx+1 > len(self.data):
			self.currentItemIdx = len(self.data) - 1
		if self.currentItemIdx < 0:
			self.currentItemIdx = 0
		if len(self.data) == 0:
			(self.sounds, word1_text, word2_text) = [], '', ''
		else:
			(self.sounds, word1_text, word2_text) = self.data[self.currentItemIdx]
		self.word1_label.text = word1_text
		self.word2_label.text = word2_text
		self.status.pop_tip()
		self.status.push_tip('%s [%s/%s]' % (os.path.basename(self.data.filename), self.currentItemIdx+1, len(self.data)))
		self.scrollbar.value = self.currentItemIdx * HSCROLL_SCALE
	def speak(self):
		"""Play audio associated with the current element"""
		if len(self.data) == 0: return
		for sound in self.data[self.currentItemIdx][0]:
			playSound(os.path.join(self.data.soundPath, sound))
	def modifyPos(self, amount, absolute=False):
		if absolute:
			self.currentItemIdx = amount
		else:
			self.currentItemIdx += amount
		self.updateWords()
	def load(self, filename):
		self.data.load(filename)
		self.status.pop_tip()
		self.status.push_tip('Loaded file %s' % os.path.basename(filename))
		self.scrollbar.set_maximum(len(self.data) * HSCROLL_SCALE)
		self.currentItemIdx = 0
		self.updateWords()
		self.scrollbar.dirty = True
		self.scrollbar.update()
	### signal handlers
	def fileDialogDone(self, result, dialog):
		if result == DLGRESULT_OK:
			filename = dialog.get_filenames()[0]
			self.load(filename)
		dialog.destroy()
	def scrollbarMoved(self):
		self.currentItemIdx = int(self.scrollbar.value / self.scrollbar.step)
		self.updateWords()

def main():
	pygame.mixer.pre_init(8000,-16,2,1024)
	
	re = Renderer()
	re.create_screen(HSIZE,VSIZE)
	re.title = "Parallel 2+"
	re.color = (234, 228, 223)
	
	mainscreen = MainScreen (re)
	mainscreen.init ()
	if len(sys.argv) > 1:
		mainscreen.load(sys.argv[1])
	if len(sys.argv) > 2:
		mainscreen.modifyPos(int(sys.argv[2])-1, absolute=True)
	
	# Start the main rendering loop.
	re.start ()

if __name__ == '__main__':
	main()
